package net.grandnature.android.soundball;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Rect;
import android.graphics.drawable.Drawable;
import android.util.AttributeSet;
import android.view.MotionEvent;
import android.view.SurfaceHolder;
import android.view.SurfaceView;

public class SoundballSurfaceView extends SurfaceView implements SurfaceHolder.Callback, Runnable {
	private static final int BALL_MAX_NUM = 6; // 最大6個まで
	private SurfaceHolder holder;
	private Drawable ballDrawable, boxDrawable;
	private Ballbox box;
	private Rect viewRect;
	private List<Soundball> balls = Collections.synchronizedList(new ArrayList<Soundball>(BALL_MAX_NUM));
	private Thread thread;

	public SoundballSurfaceView(Context context) {
		super(context);
		initialize(context);
	}

	public SoundballSurfaceView(Context context, AttributeSet attrs, int defStyle) {
		super(context, attrs, defStyle);
		initialize(context);
	}

	public SoundballSurfaceView(Context context, AttributeSet attrs) {
		super(context, attrs);
		initialize(context);
	}

	private void initialize(Context context) {
		holder = getHolder();
		holder.addCallback(this);
		holder.setFixedSize(getWidth(), getHeight());
		ballDrawable = context.getResources().getDrawable(R.drawable.ball);
		boxDrawable = context.getResources().getDrawable(R.drawable.box);
	}

	@Override
	public void surfaceChanged(SurfaceHolder holder, int format, int width, int height) {
	}

	@Override
	public void surfaceCreated(SurfaceHolder holder) {
		viewRect = new Rect(getLeft(), getTop(), getRight(), getBottom());
		box = new Ballbox(getContext(), boxDrawable, viewRect);
		thread = new Thread(this);
		thread.start();
	}

	@Override
	public void surfaceDestroyed(SurfaceHolder holder) {
		thread = null;
		releaseAllSoundballs();
	}

	@Override
	public boolean onTouchEvent(MotionEvent event) {
		float x = event.getX(), y = event.getY();
		switch (event.getAction()) {
		case MotionEvent.ACTION_DOWN:
			doActionDown(x, y);
			break;
		case MotionEvent.ACTION_MOVE:
			doActionMove(x, y);
			break;
		case MotionEvent.ACTION_UP:
			doActionUp(x, y);
			break;
		default:
			break;
		}
		return true;
	}

	// 画面をタップしたとき
	private void doActionDown(float x, float y) {
		Soundball ball = null;
		if (box.isInside(x, y) && balls.size() < BALL_MAX_NUM) {
			ball = new Soundball(getContext(), ballDrawable, viewRect);
			ball.offset((int) x, (int) y);
			balls.add(ball);
			box.playbackEffect();
		} else {
			ball = getInsideSoundball(x, y);
		}
		if (ball != null)
			ball.setHolding(true);
	}

	// ドラッグしたとき
	private void doActionMove(float x, float y) {
		Soundball ball = getHoldingSoundball();
		if (ball != null) {
			ball.set((int) x, (int) y);
			ball.notifyDistanceChanged(box);
		}
	}

	// 画面から指を放したとき
	private void doActionUp(float x, float y) {
		Soundball ball = getHoldingSoundball();
		if (ball != null) {
			ball.setHolding(false);
			if (ball.doesIntersect(box)) {
				box.playbackEffect();
				ball.stop();
				balls.remove(ball);
			}
		}
	}

	// 掴んでいるボールを調べる
	private Soundball getHoldingSoundball() {
		for (Soundball ball : balls) {
			if (ball.isHolding())
				return ball;
		}
		return null;
	}

	// タップされた座標（x,y）がいずれかボールの中にいるかを調べる
	private Soundball getInsideSoundball(float x, float y) {
		for (Soundball ball : balls) {
			if (ball.isInside(x, y))
				return ball;
		}
		return null;
	}

	// すべての音を止める
	private void releaseAllSoundballs() {
		for (Soundball ball : balls) {
			ball.release();
		}
		balls.clear();
	}

	@Override
	public void run() {
		Canvas canvas = null;
		Paint p = new Paint();
		p.setColor(Color.WHITE);
		while (thread != null) {
			try {
				canvas = holder.lockCanvas();
				if (canvas != null) {
					// 背景の描画
					canvas.drawRect(0, 0, getWidth(), getHeight(), p);
					synchronized (balls) {
						// 線の描画
						for (Soundball ball : balls) 
							ball.drawLine(canvas, box);
						// 箱の描画
						box.draw(canvas);
						// 玉の描画
						for (Soundball ball : balls) 
							ball.draw(canvas);
					}
				}
			} finally {
				if (canvas != null)
					holder.unlockCanvasAndPost(canvas);
			}
		}
	}
}